/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <folly/lang/UncaughtExceptions.h>

#if defined(__GLIBCXX__) || defined(_LIBCPP_VERSION)

namespace __cxxabiv1 {
struct __cxa_eh_globals {
  void* caught_exceptions_;
  unsigned int uncaught_exceptions_;
};
extern "C" __cxa_eh_globals* __cxa_get_globals();
} // namespace __cxxabiv1

#endif

namespace folly {

namespace detail {

unsigned int* uncaught_exceptions_ptr() noexcept {
  assert(kIsGlibcxx || kIsLibcpp);
#if defined(__GLIBCXX__) || defined(_LIBCPP_VERSION)
  return &__cxxabiv1::__cxa_get_globals()->uncaught_exceptions_;
#endif
  return nullptr;
}

} // namespace detail

} // namespace folly
